/*
 * VectorSet.hpp
 *
 *  Created on: Jun 2, 2010
 *      Author: crueger
 */

#ifndef VECTORSET_HPP_
#define VECTORSET_HPP_

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <functional>
#include <algorithm>
#include <limits>
#include <boost/bind.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/lambda/lambda.hpp>

/**
 * A simple mixin to give any STL conforming structure fast Vector abilities
 *
 * TODO: make this work for maps
 */

#include "Vector.hpp"

#include <vector>
#include "RealSpaceMatrix.hpp"

// this tests, whether we actually have a Vector
template <class V>
struct is_vector{};

template <>
struct is_vector<Vector>{
  typedef void wrong_type;
};

template <class Set>
class VectorSet : public Set
{
  // when our set carries something besides a vector this will produce an error
  typedef typename is_vector<typename Set::value_type>::wrong_type check_for_vector;
public:
  // typedefs for STL conforming structure
  typedef typename Set::iterator iterator;
  typedef typename Set::const_iterator const_iterator;

  VectorSet(){}
  VectorSet(const Set &_set) : Set(_set) {}
  virtual ~VectorSet(){}

  /**
   * transform all Vectors within this set by a RealSpaceMatrix
   */
  void transform(const RealSpaceMatrix &M){
    // this is needed to allow template lookup
    std::transform(this->begin(),this->end(),this->begin(),
        boost::bind(static_cast<Vector(*)(const RealSpaceMatrix&,const Vector&)>(operator*), M, _1));
  }

  /**
   * translate all Vectors within this set by a specified amount
   */
  void translate(const Vector &translater){
    // this is needed to allow template lookup
    std::transform(this->begin(),this->end(),this->begin(),
        std::bind1st(std::plus<Vector>(),translater));
  }

  /** Return the Vector that is closest to \a point.
   *
   * @param point
   * @return
   */
  const Vector minDistance(const Vector &point){
    if(!this->size())
      return zeroVec;
    translate(-1.*point);
//    std::transform(this->begin(),this->end(),
//        this->begin(),
//        std::bind2nd(std::mem_fun_ref(&Vector::operator-),point));
    return *min_element(this->begin(),this->end(),
//        boost::lambda::_1 < boost::lambda::_2);
        (boost::lambda::bind(&Vector::NormSquared,boost::lambda::_1) < boost::lambda::bind(&Vector::NormSquared,boost::lambda::_2)));
  }
};

// allows simpler definition of VectorSets
#define VECTORSET(container_type) VectorSet<container_type<Vector> >

#endif /* VECTORSET_HPP_ */
